#include "display_basic.h"
#include "vend.h"

void delay(u16 ms) {
	/* delay routine written in assembly so we know what we're really getting.
	 * each inner loop should take ~1 ms to execute.
	 * 15 cycles * (1/4.9152Mhz) * 327 = 0.997 ms + a little bit on the fringes.
	 *
	 * XXX - how do we know gcc isn't optimising this? it seems to optimise after
	 * parsing C -> asm, but before asm -> machine code.
	 */
	asm volatile ("pshx\npsha\npshb\n"); /* save registers */
	asm volatile ("ldx %0\n" :: "m" (ms));
	asm volatile (
		"loop:\n"
		"	dex\n"                  /* 3 */
		"	beq out\n"              /* 3 */
		"	ldd #327\n"             /* 3 */
		"inner_loop:\n" /* 15 cycles each */
		"	cpd #0x0000\n"          /* 5 */
		"	beq inner_loop_end\n"   /* 3 */
		"	subd #0x0001\n"         /* 4 */
		"	bra inner_loop\n"       /* 3 */
		"inner_loop_end:\n"
		"	bra loop\n"             /* 3 */
		"out:\n"
		"	pulb\n"
		"	pula\n"
		"	pulx\n");
}

void my_strncpy(char* dst, char* src, u8 max_size) {
	u8 i;
	for (i = 0; src[i] && i < max_size; i++) dst[i] = src[i];
	if (src[i] == 0 && i < max_size) dst[i] = 0; /* null terminator */
}

void my_memcpy(char* dst, char* src, u8 size) {
	u8 i = 0;
	for (i = 0; i < size; i++) dst[i] = src[i];
}
